#-*-coding:utf-8;-*-
from ctypes import CDLL,CFUNCTYPE,c_char_p,c_int,c_ubyte,c_uint16,c_uint32,c_uint8,c_void_p,POINTER
from json import load
from os.path import dirname,join
from sys import maxsize
if maxsize>0x100000000:
    LIBRARY=CDLL(load(open(join(dirname(__file__),"config.json"),"r",encoding="utf-8"))["x64"])
else:
    LIBRARY=CDLL(load(open(join(dirname(__file__),"config.json"),"r",encoding="utf-8"))["x86"])
rtlsdr_read_async_cb_t=CFUNCTYPE(None,POINTER(c_ubyte),c_uint32,c_void_p)
LIBRARY.rtlsdr_get_device_count.argtypes=()
LIBRARY.rtlsdr_get_device_count.restype=c_uint32
LIBRARY.rtlsdr_get_device_name.argtypes=(c_uint32,)
LIBRARY.rtlsdr_get_device_name.restype=c_char_p
LIBRARY.rtlsdr_get_device_usb_strings.argtypes=(c_uint32,c_char_p,c_char_p,c_char_p)
LIBRARY.rtlsdr_get_device_usb_strings.restype=c_int
LIBRARY.rtlsdr_get_index_by_serial.argtypes=(c_char_p,)
LIBRARY.rtlsdr_get_index_by_serial.restype=c_int
LIBRARY.rtlsdr_open.argtypes=(POINTER(c_void_p),c_uint32)
LIBRARY.rtlsdr_open.restype=c_int
LIBRARY.rtlsdr_close.argtypes=(c_void_p,)
LIBRARY.rtlsdr_close.restype=c_int
LIBRARY.rtlsdr_set_xtal_freq.argtypes=(c_void_p,c_uint32,c_uint32)
LIBRARY.rtlsdr_set_xtal_freq.restype=c_int
LIBRARY.rtlsdr_get_xtal_freq.argtypes=(c_void_p,POINTER(c_uint32),POINTER(c_uint32))
LIBRARY.rtlsdr_get_xtal_freq.restype=c_int
LIBRARY.rtlsdr_get_usb_strings.argtypes=(c_void_p,c_char_p,c_char_p,c_char_p)
LIBRARY.rtlsdr_get_usb_strings.restype=c_int
LIBRARY.rtlsdr_write_eeprom.argtypes=(c_void_p,POINTER(c_uint8),c_uint8,c_uint16)
LIBRARY.rtlsdr_write_eeprom.restype=c_int
LIBRARY.rtlsdr_read_eeprom.argtypes=(c_void_p,POINTER(c_uint8),c_uint8,c_uint16)
LIBRARY.rtlsdr_read_eeprom.restype=c_int
LIBRARY.rtlsdr_set_center_freq.argtypes=(c_void_p,c_uint32)
LIBRARY.rtlsdr_set_center_freq.restype=c_int
LIBRARY.rtlsdr_get_center_freq.argtypes=(c_void_p,)
LIBRARY.rtlsdr_get_center_freq.restype=c_uint32
LIBRARY.rtlsdr_set_freq_correction.argtypes=(c_void_p,c_int)
LIBRARY.rtlsdr_set_freq_correction.restype=c_int
LIBRARY.rtlsdr_get_freq_correction.argtypes=(c_void_p,)
LIBRARY.rtlsdr_get_freq_correction.restype=c_int
LIBRARY.rtlsdr_get_tuner_type.argtypes=(c_void_p,)
LIBRARY.rtlsdr_get_tuner_type.restype=c_int
LIBRARY.rtlsdr_get_tuner_gains.argtypes=(c_void_p,POINTER(c_int))
LIBRARY.rtlsdr_get_tuner_gains.restype=c_int
LIBRARY.rtlsdr_set_tuner_gain.argtypes=(c_void_p,c_int)
LIBRARY.rtlsdr_set_tuner_gain.restype=c_int
LIBRARY.rtlsdr_set_tuner_bandwidth.argtypes=(c_void_p,c_uint32)
LIBRARY.rtlsdr_set_tuner_bandwidth.restype=c_int
LIBRARY.rtlsdr_get_tuner_gain.argtypes=(c_void_p,)
LIBRARY.rtlsdr_get_tuner_gain.restype=c_int
LIBRARY.rtlsdr_set_tuner_if_gain.argtypes=(c_void_p,c_int,c_int)
LIBRARY.rtlsdr_set_tuner_if_gain.restype=c_int
LIBRARY.rtlsdr_set_tuner_gain_mode.argtypes=(c_void_p,c_int)
LIBRARY.rtlsdr_set_tuner_gain_mode.restype=c_int
LIBRARY.rtlsdr_set_sample_rate.argtypes=(c_void_p,c_uint32)
LIBRARY.rtlsdr_set_sample_rate.restype=c_int
LIBRARY.rtlsdr_get_sample_rate.argtypes=(c_void_p,)
LIBRARY.rtlsdr_get_sample_rate.restype=c_uint32
LIBRARY.rtlsdr_set_testmode.argtypes=(c_void_p,c_int)
LIBRARY.rtlsdr_set_testmode.restype=c_int
LIBRARY.rtlsdr_set_agc_mode.argtypes=(c_void_p,c_int)
LIBRARY.rtlsdr_set_agc_mode.restype=c_int
LIBRARY.rtlsdr_set_direct_sampling.argtypes=(c_void_p,c_int)
LIBRARY.rtlsdr_set_direct_sampling.restype=c_int
LIBRARY.rtlsdr_get_direct_sampling.argtypes=(c_void_p,)
LIBRARY.rtlsdr_get_direct_sampling.restype=c_int
LIBRARY.rtlsdr_set_offset_tuning.argtypes=(c_void_p,c_int)
LIBRARY.rtlsdr_set_offset_tuning.restype=c_int
LIBRARY.rtlsdr_get_offset_tuning.argtypes=(c_void_p,)
LIBRARY.rtlsdr_get_offset_tuning.restype=c_int
LIBRARY.rtlsdr_reset_buffer.argtypes=(c_void_p,)
LIBRARY.rtlsdr_reset_buffer.restype=c_int
LIBRARY.rtlsdr_read_sync.argtypes=(c_void_p,c_void_p,c_int,POINTER(c_int))
LIBRARY.rtlsdr_read_sync.restype=c_int
LIBRARY.rtlsdr_wait_async.argtypes=(c_void_p,rtlsdr_read_async_cb_t,c_void_p)
LIBRARY.rtlsdr_wait_async.restype=c_int
LIBRARY.rtlsdr_read_async.argtypes=(c_void_p,rtlsdr_read_async_cb_t,c_void_p,c_uint32,c_uint32)
LIBRARY.rtlsdr_read_async.restype=c_int
LIBRARY.rtlsdr_cancel_async.argtypes=(c_void_p,)
LIBRARY.rtlsdr_cancel_async.restype=c_int
LIBRARY.rtlsdr_set_bias_tee.argtypes=(c_void_p,c_int)
LIBRARY.rtlsdr_set_bias_tee.restype=c_int
LIBRARY.rtlsdr_set_bias_tee_gpio.argtypes=(c_void_p,c_int,c_int)
LIBRARY.rtlsdr_set_bias_tee_gpio.restype=c_int


#-*-coding:utf-8;-*-
from .util import rtlsdr_get_device_count,rtlsdr_get_device_name,rtlsdr_get_device_usb_strings,rtlsdr_get_index_by_serial
from .RTLSDR import rtlsdr_read_async_cb_t,rtlsdr_open,rtlsdr_close,rtlsdr_set_xtal_freq,rtlsdr_get_xtal_freq,rtlsdr_get_usb_strings,rtlsdr_write_eeprom,rtlsdr_read_eeprom,rtlsdr_set_center_freq,rtlsdr_get_center_freq,rtlsdr_set_freq_correction,rtlsdr_get_freq_correction,rtlsdr_get_tuner_type,rtlsdr_get_tuner_gains,rtlsdr_set_tuner_gain,rtlsdr_set_tuner_bandwidth,rtlsdr_get_tuner_gain,rtlsdr_set_tuner_if_gain,rtlsdr_set_tuner_gain_mode,rtlsdr_set_sample_rate,rtlsdr_get_sample_rate,rtlsdr_set_testmode,rtlsdr_set_agc_mode,rtlsdr_set_direct_sampling,rtlsdr_get_direct_sampling,rtlsdr_set_offset_tuning,rtlsdr_get_offset_tuning,rtlsdr_reset_buffer,rtlsdr_read_sync,rtlsdr_wait_async,rtlsdr_read_async,rtlsdr_cancel_async,rtlsdr_set_bias_tee,rtlsdr_set_bias_tee_gpio
from .const import rtlsdr_ds_mode,rtlsdr_tuner


from setuptools import setup
setup(
    name="RTL-SDR",
    version="1.0.0",
    author="Enbuging",
    author_email="electricfan@yeah.net",
    license="MIT",
    description="A wrapper for rtlsdr.dll, which can be used to access RTL2832U by software defined radio programs on Microsoft Windows platform.",
    keywords=["radio","SDR","RTL-SDR","RTL2832U","rtlsdr.dll"],
    platforms=["Windows"],
    package_data={
        "rtlsdr":["config.json"]
    },
    install_requires=["numpy>=1.21"],
    packages=["rtlsdr"]
)


#-*-coding:utf-8;-*-
from enum import IntEnum
class rtlsdr_ds_mode(IntEnum):
    RTLSDR_DS_IQ=0
    RTLSDR_DS_I=1
    RTLSDR_DS_Q=2
class rtlsdr_tuner(IntEnum):
    RTLSDR_TUNER_UNKNOWN=0
    RTLSDR_TUNER_E4000=1
    RTLSDR_TUNER_FC0012=2
    RTLSDR_TUNER_FC0013=3
    RTLSDR_TUNER_FC2580=4
    RTLSDR_TUNER_R820T=5
    RTLSDR_TUNER_R828D=6


#-*-coding:utf-8;-*-
from ctypes import cast,c_char,c_char_p
from typing import Tuple
from .core import LIBRARY
rtlsdr_get_device_count=LIBRARY.rtlsdr_get_device_count
rtlsdr_get_device_name=LIBRARY.rtlsdr_get_device_name
def rtlsdr_get_device_usb_strings(index:int)->Tuple[int,bytes,bytes,bytes]:
    manufact=(c_char*256)()
    product=(c_char*256)()
    serial=(c_char*256)()
    returnCode=LIBRARY.rtlsdr_get_device_usb_strings(index,cast(manufact,c_char_p),cast(product,c_char_p),cast(serial,c_char_p))
    if returnCode:
        return returnCode,b"",b"",b""
    else:
        return returnCode,manufact.value,product.value,serial.value
rtlsdr_get_index_by_serial=LIBRARY.rtlsdr_get_index_by_serial


#-*-coding:utf-8;-*-
from ctypes import byref,cast,c_char,c_char_p,c_int,c_ubyte,c_uint32,c_uint8,c_void_p,POINTER
from typing import Callable,Tuple,Union
from numpy import ubyte
from numpy.ctypeslib import as_array
from numpy.typing import NDArray
from .core import LIBRARY,rtlsdr_read_async_cb_t
from .const import rtlsdr_tuner
def rtlsdr_open(index:int)->Tuple[int,Union[int,None]]:
    dev=c_void_p()
    return LIBRARY.rtlsdr_open(byref(dev),index),dev.value
rtlsdr_close=LIBRARY.rtlsdr_close
rtlsdr_set_xtal_freq=LIBRARY.rtlsdr_set_xtal_freq
def rtlsdr_get_xtal_freq(dev:Union[int,None])->Tuple[int,int,int]:
    rtl_freq=c_uint32()
    tuner_freq=c_uint32()
    returnCode=LIBRARY.rtlsdr_get_xtal_freq(dev,byref(rtl_freq),byref(tuner_freq))
    if returnCode:
        return returnCode,-1,-1
    else:
        return returnCode,rtl_freq.value,tuner_freq.value
def rtlsdr_get_usb_strings(dev:Union[int,None])->Tuple[int,bytes,bytes,bytes]:
    manufact=(c_char*256)()
    product=(c_char*256)()
    serial=(c_char*256)()
    returnCode=LIBRARY.rtlsdr_get_usb_strings(dev,cast(manufact,c_char_p),cast(product,c_char_p),cast(serial,c_char_p))
    if returnCode:
        return returnCode,b"",b"",b""
    else:
        return returnCode,manufact.value,product.value,serial.value
def rtlsdr_write_eeprom(dev:Union[int,None],data:bytes,offset:int)->int:
    return LIBRARY.rtlsdr_write_eeprom(dev,cast((c_uint8*len(data))(*data),POINTER(c_uint8)),offset,len(data))
def rtlsdr_read_eeprom(dev:Union[int,None],offset:int,len:int)->Tuple[int,bytes]:
    data=(c_uint8*len)()
    returnCode=LIBRARY.rtlsdr_read_eeprom(dev,cast(data,POINTER(c_uint8)),offset,len)
    if returnCode:
        return returnCode,b""
    else:
        return returnCode,bytes(data[:])
rtlsdr_set_center_freq=LIBRARY.rtlsdr_set_center_freq
rtlsdr_get_center_freq=LIBRARY.rtlsdr_get_center_freq
rtlsdr_set_freq_correction=LIBRARY.rtlsdr_set_freq_correction
rtlsdr_get_freq_correction=LIBRARY.rtlsdr_get_freq_correction
def rtlsdr_get_tuner_type(dev:Union[int,None])->rtlsdr_tuner:
    return rtlsdr_tuner(LIBRARY.rtlsdr_get_tuner_type(dev))
def rtlsdr_get_tuner_gains(dev:Union[int,None])->Tuple[int,Tuple[int,...]]:
    returnCode=LIBRARY.rtlsdr_get_tuner_gains(dev,POINTER(c_int)())
    if returnCode>0:
        gains=(c_int*returnCode)()
        returnCode=LIBRARY.rtlsdr_get_tuner_gains(dev,cast(gains,POINTER(c_int)))
        if returnCode>0:
            return returnCode,(*gains[:returnCode],)
        else:
            return returnCode,()
    else:
        return returnCode,()
rtlsdr_set_tuner_gain=LIBRARY.rtlsdr_set_tuner_gain
rtlsdr_set_tuner_bandwidth=LIBRARY.rtlsdr_set_tuner_bandwidth
rtlsdr_get_tuner_gain=LIBRARY.rtlsdr_get_tuner_gain
rtlsdr_set_tuner_if_gain=LIBRARY.rtlsdr_set_tuner_if_gain
rtlsdr_set_tuner_gain_mode=LIBRARY.rtlsdr_set_tuner_gain_mode
rtlsdr_set_sample_rate=LIBRARY.rtlsdr_set_sample_rate
rtlsdr_get_sample_rate=LIBRARY.rtlsdr_get_sample_rate
rtlsdr_set_testmode=LIBRARY.rtlsdr_set_testmode
rtlsdr_set_agc_mode=LIBRARY.rtlsdr_set_agc_mode
rtlsdr_set_direct_sampling=LIBRARY.rtlsdr_set_direct_sampling
rtlsdr_get_direct_sampling=LIBRARY.rtlsdr_get_direct_sampling
rtlsdr_set_offset_tuning=LIBRARY.rtlsdr_set_offset_tuning
rtlsdr_get_offset_tuning=LIBRARY.rtlsdr_get_offset_tuning
rtlsdr_reset_buffer=LIBRARY.rtlsdr_reset_buffer
def rtlsdr_read_sync(dev:Union[int,None],len:int)->Tuple[int,NDArray[ubyte]]:
    buf=(c_ubyte*len)()
    n_read=c_int()
    return LIBRARY.rtlsdr_read_sync(dev,byref(buf),len,byref(n_read)),as_array(cast(buf,POINTER(c_ubyte)),(n_read.value,))
def rtlsdr_wait_async(dev:Union[int,None],ctx:Union[int,None])->Callable[[Callable[[NDArray[ubyte],Union[int,None]],None]],Union[rtlsdr_read_async_cb_t,None]]:
    def real_rtlsdr_wait_async(cb:Callable[[NDArray[ubyte],Union[int,None]],None])->Union[rtlsdr_read_async_cb_t,None]:
        real_cb=rtlsdr_read_async_cb_t(lambda buf,len,ctx:cb(as_array(buf,(len,)),ctx))
        if not LIBRARY.rtlsdr_wait_async(dev,real_cb,ctx):
            return real_cb
    return real_rtlsdr_wait_async
def rtlsdr_read_async(dev:Union[int,None],ctx:Union[int,None],buf_num:int,buf_len:int)->Callable[[Callable[[NDArray[ubyte],Union[int,None]],None]],Union[rtlsdr_read_async_cb_t,None]]:
    def real_rtlsdr_read_async(cb:Callable[[NDArray[ubyte],Union[int,None]],None])->Union[rtlsdr_read_async_cb_t,None]:
        real_cb=rtlsdr_read_async_cb_t(lambda buf,len,ctx:cb(as_array(buf,(len,)),ctx))
        if not LIBRARY.rtlsdr_read_async(dev,real_cb,ctx,buf_num,buf_len):
            return real_cb
    return real_rtlsdr_read_async
rtlsdr_cancel_async=LIBRARY.rtlsdr_cancel_async
rtlsdr_set_bias_tee=LIBRARY.rtlsdr_set_bias_tee
rtlsdr_set_bias_tee_gpio=LIBRARY.rtlsdr_set_bias_tee_gpio